import { Expression, Column, AllExpression, ExpressionTypeOf } from "./Expressions";
import { Query } from "./Queries/Query";
import { AnyType, BooleanType, GetInType, GetOutType, MapOutType, Record, Type } from './Types';
export declare function fromItemTypes<TColumns extends HardRow>(fromItem: FromItem<TColumns>): TColumns;
export declare function fromItemTypes<TColumns extends HardRow, TColumnNames extends keyof TColumns>(fromItem: FromItem<TColumns>, columns: TColumnNames[]): {
    [TName in TColumnNames]: TColumns[TName];
};
export interface Row {
    [name: string]: any;
}
export interface HardRow {
    [name: string]: AnyType;
}
export interface Columns {
    [name: string]: any;
}
export declare type RowToColumns<TColumns extends HardRow> = {
    [TName in keyof TColumns]: Column<TName, TColumns[TName]>;
};
export declare type ColumnsToRow<TColumns extends Columns> = {
    [TName in keyof TColumns]: ExpressionTypeOf<TColumns[TName]>;
};
export declare type ColumnsToOutRow<TColumns extends Columns> = MapOutType<ColumnsToRow<TColumns>>;
export declare class FromFactor {
    _brand: "FromFactor";
    static getAllFromFactors(factor: FromFactor): FromItem<any>[];
    static crossJoin(factor1: FromFactor | undefined, factor2: FromFactor | undefined): FromFactor | undefined;
}
export declare abstract class FromFactorAbstractJoin extends FromFactor {
    readonly leftArg: FromFactor;
    readonly rightArg: FromFactor;
    constructor(leftArg: FromFactor, rightArg: FromFactor);
}
export declare abstract class FromFactorAbstractConditionalJoin extends FromFactorAbstractJoin {
    readonly joinCondition: Expression<BooleanType>;
    constructor(leftArg: FromFactor, rightArg: FromFactor, joinCondition: Expression<BooleanType>);
    abstract getType(): string;
}
export declare class FromFactorLeftJoin extends FromFactorAbstractConditionalJoin {
    getType(): string;
}
export declare class FromFactorFullJoin extends FromFactorAbstractConditionalJoin {
    getType(): string;
}
export declare class FromFactorInnerJoin extends FromFactorAbstractConditionalJoin {
    getType(): string;
}
export declare class FromFactorCrossJoin extends FromFactorAbstractJoin {
    getType(): string;
}
export declare type FromItemToOutRow<TFromItem extends FromItem<any>> = {
    [TName in keyof TFromItem["$columns"]]: GetOutType<ExpressionTypeOf<TFromItem["$columns"][TName]>>;
};
export declare type FromItemToInRow<TFromItem extends FromItem<any>> = {
    [TName in keyof TFromItem["$columns"]]: GetInType<ExpressionTypeOf<TFromItem["$columns"][TName]>>;
};
export declare abstract class FromItem<TColumns extends HardRow> extends FromFactor {
    private readonly castToColumns;
    [key: string]: any;
    readonly $columns: RowToColumns<TColumns>;
    readonly $all: AllExpression<TColumns>;
    constructor(columns: RowToColumns<TColumns>, castToColumns: boolean);
    as(name: string): FromItemCtor<TColumns>;
    asNullable(): FromItemCtor<{
        [TKey in keyof TColumns]: Type<GetInType<TColumns[TKey]>, GetOutType<TColumns[TKey]> | null, TColumns[TKey]["_brand"]>;
    }>;
    asExpression(): Expression<Record<TColumns>>;
}
export declare class NullableFromItem<TColumns extends Row> extends FromItem<TColumns> {
    readonly fromItem: FromItem<TColumns>;
    constructor(columns: RowToColumns<TColumns>, fromItem: FromItem<TColumns>, castToColumns: boolean);
}
export declare function isCastToColumns(fromItem: FromItem<any>): boolean;
export declare function getColumn(fromItem: FromItem<any>, column: string): Column<string, AnyType>;
export declare class NamedFromItem<TColumns extends Row> extends FromItem<TColumns> {
    readonly $name: string;
    readonly fromItem: FromItem<TColumns>;
    constructor($name: string, columns: RowToColumns<TColumns>, fromItem: FromItem<TColumns>, castToColumns: boolean);
}
export declare class QueryFromItem<TColumns extends HardRow> extends FromItem<TColumns> {
    readonly $name: string;
    readonly query: Query<TColumns, any>;
    constructor($name: string, columns: RowToColumns<TColumns>, query: Query<TColumns, any>, castToColumns: boolean);
}
export declare type FromItemCtor<TColumns extends Row> = FromItem<TColumns> & RowToColumns<TColumns>;
