Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryFromItem = exports.NamedFromItem = exports.getColumn = exports.isCastToColumns = exports.NullableFromItem = exports.FromItem = exports.FromFactorCrossJoin = exports.FromFactorInnerJoin = exports.FromFactorFullJoin = exports.FromFactorLeftJoin = exports.FromFactorAbstractConditionalJoin = exports.FromFactorAbstractJoin = exports.FromFactor = exports.fromItemTypes = void 0;
const Expressions_1 = require("./Expressions");
const Helpers_1 = require("../Helpers");
function fromItemTypes(fromItem, columns = []) {
    return (0, Helpers_1.toObject)((0, Helpers_1.objectValues)(fromItem.$columns).filter(c => columns.indexOf(c.name) !== -1), k => k.name, k => k.type);
}
exports.fromItemTypes = fromItemTypes;
class FromFactor {
    static getAllFromFactors(factor) {
        if (factor instanceof FromItem)
            return [factor];
        else if (factor instanceof FromFactorAbstractJoin) {
            return FromFactor.getAllFromFactors(factor.leftArg).concat(FromFactor.getAllFromFactors(factor.rightArg));
        }
        throw new Error("Unsupported from factor");
    }
    static crossJoin(factor1, factor2) {
        if (!factor1)
            return factor2;
        if (!factor2)
            return factor1;
        return new FromFactorCrossJoin(factor1, factor2);
    }
}
exports.FromFactor = FromFactor;
class FromFactorAbstractJoin extends FromFactor {
    constructor(leftArg, rightArg) {
        super();
        this.leftArg = leftArg;
        this.rightArg = rightArg;
    }
}
exports.FromFactorAbstractJoin = FromFactorAbstractJoin;
class FromFactorAbstractConditionalJoin extends FromFactorAbstractJoin {
    constructor(leftArg, rightArg, joinCondition) {
        super(leftArg, rightArg);
        this.joinCondition = joinCondition;
    }
}
exports.FromFactorAbstractConditionalJoin = FromFactorAbstractConditionalJoin;
class FromFactorLeftJoin extends FromFactorAbstractConditionalJoin {
    getType() { return "left"; }
}
exports.FromFactorLeftJoin = FromFactorLeftJoin;
;
class FromFactorFullJoin extends FromFactorAbstractConditionalJoin {
    getType() { return "full"; }
}
exports.FromFactorFullJoin = FromFactorFullJoin;
;
class FromFactorInnerJoin extends FromFactorAbstractConditionalJoin {
    getType() { return "inner"; }
}
exports.FromFactorInnerJoin = FromFactorInnerJoin;
;
class FromFactorCrossJoin extends FromFactorAbstractJoin {
    getType() { return "cross"; }
}
exports.FromFactorCrossJoin = FromFactorCrossJoin;
;
class FromItem extends FromFactor {
    constructor(columns, castToColumns) {
        super();
        this.castToColumns = castToColumns;
        this.$all = new Expressions_1.AllExpression(this);
        this.$columns = columns;
        for (const [name, col] of (0, Helpers_1.objectEntries)(columns)) {
            if (!(name in this))
                this[name] = col;
        }
    }
    as(name) {
        const setters = [];
        const columns = (0, Helpers_1.objectValues)(this.$columns)
            .map((col) => new Expressions_1.ColumnBoundToExpression(col, s => setters.push(s)));
        const result = new NamedFromItem(name, (0, Helpers_1.toObject)(columns, c => c.name), this, false);
        for (const s of setters)
            s(result);
        return result;
    }
    asNullable() {
        return this; //TODO!!!
        //const nullableColumns = toObject(objectEntries(this.$columns), k => k[0], k => k[1].cast(k[1].type.orNull()));
        //return new NullableFromItem(nullableColumns, this.castToColumns) as any;
    }
    asExpression() {
        return new Expressions_1.FromItemExpression(this);
    }
}
exports.FromItem = FromItem;
class NullableFromItem extends FromItem {
    constructor(columns, fromItem, castToColumns) {
        super(columns, castToColumns);
        this.fromItem = fromItem;
    }
}
exports.NullableFromItem = NullableFromItem;
function isCastToColumns(fromItem) {
    return fromItem["castToColumns"];
}
exports.isCastToColumns = isCastToColumns;
function getColumn(fromItem, column) {
    const result = fromItem.$columns[column];
    if (!result)
        throw new Error(`Column '${column}' does not exist on table '${fromItem}'.`);
    return result;
}
exports.getColumn = getColumn;
class NamedFromItem extends FromItem {
    constructor($name, columns, fromItem, castToColumns) {
        super(columns, castToColumns);
        this.$name = $name;
        this.fromItem = fromItem;
    }
}
exports.NamedFromItem = NamedFromItem;
class QueryFromItem extends FromItem {
    constructor($name, columns, query, castToColumns) {
        super(columns, castToColumns);
        this.$name = $name;
        this.query = query;
    }
}
exports.QueryFromItem = QueryFromItem;
/* TODO, for recursion
export class BindableFromItem<TColumns extends ImplicitColumns> extends FromItem<TColumns> {
    public bind(fromItem: FromItem<TColumns>): void;
    public bindAndOverrideColumnNames(view: FromItem<any>): void;
}

type BindableViewCtor<TColumns extends ImplicitColumns> = BindableFromItem<TColumns> & ImplicitColumnsToColumns<TColumns>;

export function view<TColumnsWithTypes extends { [columnName: string]: Columns.ColumnDescription<any> }>(
        tableName: string, columns: TColumnsWithTypes):
    BindableViewCtor<ColumnsWithTypesToImplicit<TColumnsWithTypes>> {
    
    return null!;
}
*/ 
//# sourceMappingURL=FromFactor.js.map