import { Expression, Column, NamedExpression, NamedExpressionNameOf, ExpressionTypeOf, AllExpression } from "../Expressions";
import { FromItem, HardRow } from "../FromFactor";
import { AnyType } from "../Types";
/**
 * Used to report error messages through overloading.
 */
export interface ErrorMessage<TMsg> {
    msg: TMsg;
}
/**
 * An helper for mixins.
 * @private
 */
export declare type Constructable<T> = new (...args: any[]) => T;
/**
 * If `expression` is a string, it returns the column from `fromItem` that has the name `expression` (or throws an exception).
 * Otherwise, it ensures that `expression` is an `Expression` and returns it.
 * @private
 */
export declare function resolveColumnReference<TExpr extends Expression<any>, TFromTblCols extends HardRow>(fromItem: FromItem<TFromTblCols> | undefined, expression: TExpr | keyof TFromTblCols): TExpr | Column<string, AnyType>;
/**
 * An helper method for select or returning.
 * @private
 */
export declare function handleSelect<TFromTblCols extends HardRow>(fromItem: FromItem<TFromTblCols> | undefined, selectedExpressions: ((keyof TFromTblCols) | NmdExpr | AllExpression<object>)[], allReturningColumns: {
    [exprName: string]: NamedExpression<string, AnyType>;
}, allSelectedExpressions: (NamedExpression<string, AnyType> | AllExpression<object>)[]): void;
/**
 * A functor that forces typescript to resolve `|` and `&` operators.
 * @private
 */
export declare type Simplify<T> = {
    [TKey in keyof T]: T[TKey];
};
/**
 * A short form for `NamedExpression<any, any>`.
 * @private
 */
export declare type NmdExpr = NamedExpression<string, AnyType>;
/**
 *
 * @private
 */
export declare type NmdExprToRow<TCol extends NmdExpr> = {
    [TKey in NamedExpressionNameOf<TCol>]: ExpressionTypeOf<TCol>;
};
