Object.defineProperty(exports, "__esModule", { value: true });
exports.handleSelect = exports.resolveColumnReference = void 0;
const Expressions_1 = require("../Expressions");
const Helpers_1 = require("../../Helpers");
/**
 * If `expression` is a string, it returns the column from `fromItem` that has the name `expression` (or throws an exception).
 * Otherwise, it ensures that `expression` is an `Expression` and returns it.
 * @private
 */
function resolveColumnReference(fromItem, expression) {
    if (expression instanceof Expressions_1.Expression)
        return expression;
    if (typeof expression !== "string")
        throw new Error(`Expression must be either of type string or of type Expression, but was '${expression}'.`);
    if (!fromItem) {
        throw new Error(`Columns cannot be referenced by string if no table is specified to select from. `
            + `Use method 'from' before.`);
    }
    const column = fromItem.$columns[expression];
    if (!column)
        throw new Error(`Column with name '${expression}' does not exist on last table that has been specified in from.`);
    return column;
}
exports.resolveColumnReference = resolveColumnReference;
/**
 * An helper method for select or returning.
 * @private
 */
function handleSelect(fromItem, selectedExpressions, allReturningColumns, allSelectedExpressions) {
    const normalizedExpressions = selectedExpressions.map(arg => resolveColumnReference(fromItem, arg));
    for (const expr of normalizedExpressions) {
        if (!(expr instanceof Expressions_1.NamedExpression || expr instanceof Expressions_1.AllExpression))
            throw new Error(`Given argument '${expr}' must be a named or an all expression.`);
    }
    for (const expr of normalizedExpressions) {
        for (const col of (expr instanceof Expressions_1.AllExpression) ? (0, Helpers_1.objectValues)(expr.fromItem.$columns) : [expr])
            allReturningColumns[col.name] = col;
        allSelectedExpressions.push(expr);
    }
}
exports.handleSelect = handleSelect;
//# sourceMappingURL=Common.js.map