import { FromItem, Row, FromFactor, HardRow } from "../FromFactor";
import { Table } from "../Table";
import { AllExpression, NamedExpression, Expression, NamedExpressionNameOf, MapExpressionOrInputValue } from "../Expressions";
import { Query, NoColumnsSelected, MoreThanOneColumnSelected, SingleColumn } from "./Query";
import { NmdExpr, NmdExprToRow, ErrorMessage } from "./Common";
import { JoinConditionBuilder } from "./JoinMixin";
import { BooleanType } from "../Types";
/**
 * Creates a DELETE FROM statement.
 *
 * @param table The table to delete from.
 */
export declare function deleteFrom<T extends Row>(table: FromItem<T> & Table<any, any>): DeleteQuery<T, {}, NoColumnsSelected>;
/**
 * Represents a DELETE FROM query.
 */
export declare class DeleteQuery<TLastFromRow extends Row, TReturningRow extends Row, TSingleColumn extends SingleColumn<TReturningRow>> extends Query<TReturningRow, TSingleColumn> {
    private readonly table;
    constructor(table: FromItem<any> & Table<any, any>);
    getState(): {
        isWithRecursive: boolean;
        withItems: FromItem<any>[];
        selected: (AllExpression<any> | NamedExpression<any, any>)[];
        returningColumns: import("./Query").RowToNmdExprs<TReturningRow>;
    } & {
        table: FromItem<any> & Table<any, any>;
        using: FromFactor | undefined;
        whereCondition: Expression<BooleanType> | undefined;
    };
    /**
     * Brings an additional query into scope that can be used to specify more complex delete conditions.
     *
     * @param fromItem The query to use.
     */
    using<TFromRow extends Row>(fromItem: FromItem<TFromRow>): DeleteQuery<TFromRow, TReturningRow, TSingleColumn>;
    /** Selects all columns from the given table. */
    returning<T extends Row>(table: AllExpression<T>): DeleteQuery<TLastFromRow, TReturningRow & {
        [TName in keyof T]: T[TName];
    }, MoreThanOneColumnSelected>;
    /** Selects a single named expression. */
    returning<T1 extends NmdExpr>(this: DeleteQuery<TLastFromRow, TReturningRow, NoColumnsSelected> | void, expr1: T1): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1>, NamedExpressionNameOf<T1>>;
    /** Selects 1 named expressions. */
    returning<T1 extends NmdExpr>(expr1: T1): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1>, MoreThanOneColumnSelected>;
    /** Selects 2 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr>(expr1: T1, expr2: T2): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2>, MoreThanOneColumnSelected>;
    /** Selects 3 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3>, MoreThanOneColumnSelected>;
    /** Selects 4 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4>, MoreThanOneColumnSelected>;
    /** Selects 5 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5>, MoreThanOneColumnSelected>;
    /** Selects 6 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6>, MoreThanOneColumnSelected>;
    /** Selects 7 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr, T7 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6, expr7: T7): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6> & NmdExprToRow<T7>, MoreThanOneColumnSelected>;
    /** Selects 8 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr, T7 extends NmdExpr, T8 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6, expr7: T7, expr8: T8): DeleteQuery<TLastFromRow, TReturningRow & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6> & NmdExprToRow<T7> & NmdExprToRow<T8>, MoreThanOneColumnSelected>;
    /** Selects a single column that is currently in scope. */
    returning<TColumnName extends keyof TLastFromRow>(this: DeleteQuery<TLastFromRow, TReturningRow, NoColumnsSelected>, column1: TColumnName): DeleteQuery<TLastFromRow, TReturningRow & {
        [TName in TColumnName]: TLastFromRow[TName];
    }, TColumnName>;
    /** Selects columns that are currently in scope. */
    returning<TColumnNames extends keyof TLastFromRow>(...columns: TColumnNames[]): DeleteQuery<TLastFromRow, TReturningRow & {
        [TName in TColumnNames]: TLastFromRow[TName];
    }, MoreThanOneColumnSelected>;
    returning(this: ErrorMessage<"Expressions must have names. Use expr.as('name').">, ...expr: Expression<any>[]): "Error";
    protected _whereCondition: Expression<BooleanType> | undefined;
    protected lastFromItem: FromItem<TLastFromRow> | undefined;
    /**
    * Adds where conditions.
    * @param obj The object that defines equals expressions.
    */
    where(obj: Partial<MapExpressionOrInputValue<TLastFromRow>>): this;
    /**
    * Adds where conditions.
    * @param conditions The condition expressions.
    */
    where(...conditions: Expression<BooleanType>[]): this;
    /**
    * Adds negated where conditions.
    */
    whereNot(condition: Expression<BooleanType>, ...conditions: Expression<BooleanType>[]): this;
    protected _from: FromFactor | undefined;
    /**
    * Performs a full join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * for (row r in cur): if (joined has no row that matches r)
    * 	yield row(r, null)
    * for (row j in joined): if (cur has no row that matches j)
    * 	yield row(null, j)
    * ```
    */
    fullJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
    /**
    * Performs a left join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * for (row r in cur): if (joined has no row that matches r)
    * 	yield row(r, null)
    * ```
    */
    leftJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
    /**
    * Performs an inner join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * ```
    */
    innerJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
}
