Object.defineProperty(exports, "__esModule", { value: true });
exports.InsertQuery = exports.InsertQueryBuilder = exports.insertInto = void 0;
const Query_1 = require("./Query");
const Common_1 = require("./Common");
const Helpers_1 = require("../../Helpers");
/**
 * Creates an INSERT INTO statement.
 *
 * @param table The table to insert into.
 */
function insertInto(table) {
    return new InsertQueryBuilder(table);
}
exports.insertInto = insertInto;
/**
 * Represents an INSERT INTO statement that needs values.
 */
class InsertQueryBuilder {
    constructor(table) {
        this.table = table;
    }
    /**
     * Inserts one or more items.
     * @param items The items to be inserted.
     */
    value(...items) {
        return this.values(items);
    }
    /**
     * Inserts an array of items.
     * @param items The items to be inserted.
     */
    values(items) {
        return new InsertQuery(this.table, items);
    }
    /**
     * Uses a subquery to retrieve the items to be inserted.
     * @param query A query that returns the items to be inserted.
     */
    valuesFrom(query) {
        return new InsertQuery(this.table, query);
    }
}
exports.InsertQueryBuilder = InsertQueryBuilder;
/**
 * Represents an INSERT INTO statement.
 */
class InsertQuery extends Query_1.Query {
    constructor(table, values) {
        super();
        this.table = table;
        this.values = values;
    }
    getState() {
        return (0, Helpers_1.combine)(super.getState(), {
            table: this.table,
            values: this.values
        });
    }
    returning(...args) {
        (0, Common_1.handleSelect)(this.table, args, this.returningColumns, this.selectedExpressions);
        return this;
    }
}
exports.InsertQuery = InsertQuery;
//# sourceMappingURL=InsertQuery.js.map