import { BooleanType } from '../Types';
import { Expression, MapExpressionOrInputValue } from "../Expressions";
import { FromItem, FromFactor } from "../FromFactor";
export declare type FromFactorCtor = new (from: FromFactor, joined: FromFactor, condition: Expression<BooleanType>) => FromFactor;
export declare function doJoin<TFromItemColumns, TSelf>(self: TSelf, currentFrom: FromFactor | undefined, fromUpdater: (newFrom: FromFactor) => void, ctor: FromFactorCtor, fromItem: FromItem<any>): JoinConditionBuilder<unknown, TSelf>;
export declare class JoinConditionBuilder<TFromItemColumns, TReturn> {
    private readonly fromItem;
    private readonly onJoin;
    constructor(fromItem: FromItem<any>, onJoin: (joined: FromItem<any>, condition: Expression<BooleanType>) => TReturn);
    /**
     * Specifies the condition of the join.
     * At least one condition must be set.
     *
     * @param condition The condition.
     * @param furtherConditions More conditions.
     */
    on(condition: Expression<BooleanType>, ...furtherConditions: Expression<BooleanType>[]): TReturn;
    /**
     * Specifies the condition of the join.
     * At least one property must be set.
     *
     * @param condition An object that represents multiple equals conditions.
     *	Each property refers to a column of the joined table, its value to the expression the column must be equal to.
     */
    on(condition: Partial<MapExpressionOrInputValue<TFromItemColumns>>): TReturn;
}
