import { AnyType } from '../Types';
import { FromItem, FromItemCtor, Row } from "../FromFactor";
import { NamedExpression, AllExpression } from "../Expressions";
import { RowDescriptionToRow } from "../Table";
export declare type RowToNmdExprs<TRow extends Row> = {
    [TName in keyof TRow]: NamedExpression<TName, TRow[TName]>;
};
export interface NoColumnsSelected {
    _brand: "NoColumnSelected";
}
export interface MoreThanOneColumnSelected {
    _brand: "MoreThanOneColumnSelected";
}
export declare type SingleColumn<TSelectedCols> = NoColumnsSelected | (keyof TSelectedCols) | MoreThanOneColumnSelected;
export declare class Query<TReturningRow extends Row, TSingleColumn extends SingleColumn<TReturningRow>> {
    protected returningColumns: RowToNmdExprs<TReturningRow>;
    protected selectedExpressions: (NamedExpression<any, any> | AllExpression<any>)[];
    protected then: () => never;
    /**
     * Returns `NoColumnsSelected` if no columns are selected,
     * `MoreThanOneColumnSelected` if more than one columns are selected or
     * the name of the column if exactly one column is selected.
     */
    get singleColumn(): TSingleColumn;
    private _isWithRecursive;
    private _withItems;
    getState(): {
        isWithRecursive: boolean;
        withItems: FromItem<any>[];
        selected: (AllExpression<any> | NamedExpression<any, any>)[];
        returningColumns: RowToNmdExprs<TReturningRow>;
    };
    /**
     * Names this query and returns it as `FromItem`.
     * @param name The name of the query that will be used in the SQL statement.
     */
    as(name: string): FromItemCtor<TReturningRow>;
    /**
     * Names this query and renames its columns and returns it as `FromItem`.
     * @param name The name of the query that will be used in the SQL statement.
     * @param columns The columns to cast to.
     */
    as<TColumnsWithTypes extends {
        [columnName: string]: AnyType;
    }>(name: string, columns: TColumnsWithTypes): FromItemCtor<RowDescriptionToRow<TColumnsWithTypes>>;
}
