Object.defineProperty(exports, "__esModule", { value: true });
exports.Query = void 0;
const FromFactor_1 = require("../FromFactor");
const Expressions_1 = require("../Expressions");
const Helpers_1 = require("../../Helpers");
class Query {
    constructor() {
        this.returningColumns = {};
        this.selectedExpressions = [];
        this.then = () => { throw new Error("You probably don't want to await a query directy. Use a DbConnection to execute queries."); };
        this._isWithRecursive = false;
        this._withItems = [];
    }
    /**
     * Returns `NoColumnsSelected` if no columns are selected,
     * `MoreThanOneColumnSelected` if more than one columns are selected or
     * the name of the column if exactly one column is selected.
     */
    get singleColumn() {
        const first = this.selectedExpressions[0];
        if (this.selectedExpressions.length === 1 && !(first instanceof Expressions_1.AllExpression))
            return first.name;
        return undefined;
    }
    getState() {
        return {
            isWithRecursive: this._isWithRecursive,
            withItems: this._withItems,
            selected: this.selectedExpressions,
            returningColumns: this.returningColumns
        };
    }
    as(name, columns) {
        const newColumns = {};
        const setters = [];
        let castToColumns;
        if (!columns) {
            castToColumns = false;
            for (const nmdExpr of (0, Helpers_1.objectValues)(this.returningColumns)) {
                newColumns[nmdExpr.name] = new Expressions_1.ColumnBoundToExpression(nmdExpr, setter => setters.push(setter));
            }
        }
        else {
            castToColumns = true;
            for (const [name, type] of (0, Helpers_1.objectEntries)(columns)) {
                newColumns[name] = new Expressions_1.AsColumn(name, type, setter => setters.push(setter));
            }
        }
        const fromItem = new FromFactor_1.QueryFromItem(name, newColumns, this, castToColumns);
        for (const setter of setters)
            setter(fromItem);
        return fromItem; // fields are set in constructor
    }
}
exports.Query = Query;
//# sourceMappingURL=Query.js.map