import { IntegerType } from '../Types';
import { Query, SingleColumn } from "./Query";
import { Expression, ExpressionOrInputValue, AllExpression, NamedExpression } from "../Expressions";
import { FromItem, Row } from "../FromFactor";
export declare class RetrievalQuery<TRow extends Row, TSingleColumn extends SingleColumn<TRow>> extends Query<TRow, TSingleColumn> {
    private _limit;
    private _offset;
    getState(): {
        limit: Expression<IntegerType> | undefined;
        offset: Expression<IntegerType> | undefined;
    } & {
        isWithRecursive: boolean;
        withItems: FromItem<any>[];
        selected: (AllExpression<any> | NamedExpression<any, any>)[];
        returningColumns: import("./Query").RowToNmdExprs<TRow>;
    };
    /**
     * Sets an limit on how many rows are returned.
     * Overwrites any limit that was set before.
     *
     * @param count The limit.
     */
    limit(count: ExpressionOrInputValue<IntegerType>): this;
    /**
     * Sets how many rows are skipped.
     * Overwrites any offset that was set before.
     *
     * @param count The offset.
     */
    offset(count: ExpressionOrInputValue<IntegerType>): this;
    /**
     * Wraps this query as expression. The query must return exactly one row with exactly one column.
     */
    asExpression<TSingleColumn2 extends keyof TRow>(this: RetrievalQuery<TRow, TSingleColumn2>): Expression<TRow[TSingleColumn2]>;
}
export declare class UnionQuery<TColumns, TSingleColumn extends SingleColumn<TColumns>> extends RetrievalQuery<TColumns, TSingleColumn> {
    readonly query1: RetrievalQuery<TColumns, TSingleColumn>;
    readonly query2: RetrievalQuery<TColumns, any>;
    readonly isUnionAll: boolean;
    constructor(query1: RetrievalQuery<TColumns, TSingleColumn>, query2: RetrievalQuery<TColumns, any>, isUnionAll: boolean);
}
export declare function union<TColumns, TSingleColumn extends SingleColumn<TColumns>>(query1: RetrievalQuery<TColumns, TSingleColumn>, ...queries: RetrievalQuery<any, any>[]): RetrievalQuery<TColumns, TSingleColumn>;
export declare function unionAll<TColumns, TSingleColumn extends SingleColumn<TColumns>>(query1: RetrievalQuery<TColumns, TSingleColumn>, ...queries: RetrievalQuery<any, any>[]): RetrievalQuery<TColumns, TSingleColumn>;
