Object.defineProperty(exports, "__esModule", { value: true });
exports.unionAll = exports.union = exports.UnionQuery = exports.RetrievalQuery = void 0;
const Types_1 = require("../Types");
const Query_1 = require("./Query");
const Expressions_1 = require("../Expressions");
class RetrievalQuery extends Query_1.Query {
    getState() {
        return Object.assign({
            limit: this._limit,
            offset: this._offset
        }, super.getState());
    }
    /**
     * Sets an limit on how many rows are returned.
     * Overwrites any limit that was set before.
     *
     * @param count The limit.
     */
    limit(count) {
        this._limit = (0, Expressions_1.normalize)(Types_1.tInteger, count);
        return this;
    }
    /**
     * Sets how many rows are skipped.
     * Overwrites any offset that was set before.
     *
     * @param count The offset.
     */
    offset(count) {
        this._offset = (0, Expressions_1.normalize)(Types_1.tInteger, count);
        return this;
    }
    /**
     * Wraps this query as expression. The query must return exactly one row with exactly one column.
     */
    asExpression() {
        const column = this.returningColumns[this.singleColumn];
        return new Expressions_1.RetrievalQueryAsExpression(this, column.type);
    }
}
exports.RetrievalQuery = RetrievalQuery;
class UnionQuery extends RetrievalQuery {
    constructor(query1, query2, isUnionAll) {
        super();
        this.query1 = query1;
        this.query2 = query2;
        this.isUnionAll = isUnionAll;
        const s = query1.getState();
        this.returningColumns = s.returningColumns;
        this.selectedExpressions = s.selected;
    }
}
exports.UnionQuery = UnionQuery;
function union(query1, ...queries) {
    return queries.reduce((p, c) => new UnionQuery(p, c, false), query1);
}
exports.union = union;
function unionAll(query1, ...queries) {
    return queries.reduce((p, c) => new UnionQuery(p, c, true), query1);
}
exports.unionAll = unionAll;
//# sourceMappingURL=RetrievalQuery.js.map