import { BooleanType, AnyType, IntegerType } from '../Types';
import { Expression, NamedExpression, MapExpressionOrInputValue, AllExpression, NamedExpressionNameOf } from "../Expressions";
import { FromItem, FromFactor, Row, RowToColumns, HardRow } from "../FromFactor";
import { RetrievalQuery } from "./RetrievalQuery";
import { NoColumnsSelected, MoreThanOneColumnSelected, SingleColumn } from "./Query";
import { Ordering } from "../Ordering";
import { Simplify, NmdExpr, NmdExprToRow, ErrorMessage } from "./Common";
import { JoinConditionBuilder } from "./JoinMixin";
export declare class SelectQuery<TSelectedCols extends Row, TFromTblCols extends Row, TSingleColumn extends SingleColumn<TSelectedCols>> extends RetrievalQuery<TSelectedCols, TSingleColumn> {
    private _orderBys;
    private _havingCondition;
    private _groupBys;
    getState(): {
        orderBys: Ordering<Expression<AnyType>>[];
        whereCondition: Expression<BooleanType> | undefined;
        havingCondition: Expression<BooleanType> | undefined;
        groupBys: Expression<AnyType>[];
        from: FromFactor | undefined;
    } & {
        limit: Expression<IntegerType> | undefined;
        offset: Expression<IntegerType> | undefined;
    } & {
        isWithRecursive: boolean;
        withItems: FromItem<any>[];
        selected: (AllExpression<any> | NamedExpression<any, any>)[];
        returningColumns: import("./Query").RowToNmdExprs<TSelectedCols>;
    };
    /**
     * Selects from a table. If previous tables are already specified, they are cross joined.
     * @param table The table to select from.
     */
    from<TTableColumns extends HardRow>(table: FromItem<TTableColumns>): SelectQuery<TSelectedCols, TTableColumns, TSingleColumn>;
    /** Selects all columns from the given table. */
    select<T extends Row>(table: AllExpression<T>): SelectQuery<Simplify<TSelectedCols & {
        [TName in keyof T]: T[TName];
    }>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects a single named expression. */
    select<T1 extends NmdExpr>(this: SelectQuery<TSelectedCols, TFromTblCols, NoColumnsSelected> | void, expr1: T1): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1>>, TFromTblCols, NamedExpressionNameOf<T1>>;
    /** Selects 1 named expressions. */
    select<T1 extends NmdExpr>(expr1: T1): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 2 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr>(expr1: T1, expr2: T2): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 3 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 4 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 5 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 6 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 7 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr, T7 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6, expr7: T7): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6> & NmdExprToRow<T7>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 8 named expressions. */
    select<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr, T7 extends NmdExpr, T8 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6, expr7: T7, expr8: T8): SelectQuery<Simplify<TSelectedCols & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6> & NmdExprToRow<T7> & NmdExprToRow<T8>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects a single column that is currently in scope. */
    select<TColumnName extends keyof TFromTblCols>(this: SelectQuery<TSelectedCols, TFromTblCols, NoColumnsSelected>, column1: TColumnName): SelectQuery<Simplify<TSelectedCols & {
        [TName in TColumnName]: TFromTblCols[TName];
    }>, TFromTblCols, TColumnName>;
    /** Selects columns that are currently in scope. */
    select<TColumnNames extends keyof TFromTblCols>(...columns: TColumnNames[]): SelectQuery<Simplify<TSelectedCols & {
        [TName in TColumnNames]: TFromTblCols[TName];
    }>, TFromTblCols, MoreThanOneColumnSelected>;
    select(this: ErrorMessage<"Expressions must have names. Use expr.as('name').">, ...expr: Expression<any>[]): "Error";
    /**
     * Adds expressions to the order-by list of this query.
     * Use `expression.desc()` or `{ desc: "columnName" }` for descending order.
     * @param expressions The expressions to add to the order-by list.
     */
    orderBy(...expressions: (Ordering<(Expression<any> | keyof TFromTblCols)> | Expression<any> | keyof TFromTblCols)[]): this;
    /**
     * Adds expressions to the order-by list of this query. The `expressionSelector` can be used to refer to already selected expressions.
     * @param expressionSelector A selector that must return the expressions to add to the order-by list.
     */
    orderBy(expressionSelector: (selectedColumns: RowToColumns<TSelectedCols>) => (Ordering<Expression<any>> | Expression<any>)[] | (Ordering<Expression<any>> | Expression<any>)): this;
    /**
     * Adds having-conditions.
     * In contrast to where-conditions, having-conditions are evaluated after grouping is done.
     * @param conditionSelector
     */
    having(condition: Expression<BooleanType>, ...conditions: Expression<BooleanType>[]): this;
    /**
     * Adds having-conditions.
     * In contrast to where-conditions, having-conditions are evaluated after grouping is done.
     * @param conditionSelector
     */
    having(conditionSelector: (selectedColumns: RowToColumns<TSelectedCols>) => Expression<BooleanType>): this;
    /**
     * Adds expressions to the group-by list of this query.
     * @param expressions The expressions to add.
     */
    groupBy(...expressions: (Expression<AnyType> | keyof TFromTblCols)[]): this;
    /**
     * Adds expressions to the group-by list of this query. The `expressionSelector` can be used to refer to already selected expressions.
     * @param expressionSelector A selector that must return the expressions to add to the group-by list.
     */
    groupBy(expressionSelector: (selectedColumns: RowToColumns<TSelectedCols>) => Expression<AnyType> | Expression<AnyType>[]): this;
    protected _whereCondition: Expression<BooleanType> | undefined;
    protected lastFromItem: FromItem<TFromTblCols> | undefined;
    /**
    * Adds where conditions.
    * @param obj The object that defines equals expressions.
    */
    where(obj: Partial<MapExpressionOrInputValue<TFromTblCols>>): this;
    /**
    * Adds where conditions.
    * @param conditions The condition expressions.
    */
    where(...conditions: Expression<BooleanType>[]): this;
    /**
    * Adds negated where conditions.
    */
    whereNot(condition: Expression<BooleanType>, ...conditions: Expression<BooleanType>[]): this;
    protected _from: FromFactor | undefined;
    /**
    * Performs a full join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * for (row r in cur): if (joined has no row that matches r)
    * 	yield row(r, null)
    * for (row j in joined): if (cur has no row that matches j)
    * 	yield row(null, j)
    * ```
    */
    fullJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
    /**
    * Performs a left join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * for (row r in cur): if (joined has no row that matches r)
    * 	yield row(r, null)
    * ```
    */
    leftJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
    /**
    * Performs an inner join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * ```
    */
    innerJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
}
/**
 * Creates a SELECT query that selects from the given table.
 * @param table The table to select from.
 */
export declare function from<TTableColumns extends HardRow>(table: FromItem<TTableColumns>): SelectQuery<{}, TTableColumns, NoColumnsSelected>;
export declare const select: {
    <T extends Row>(table: AllExpression<T>): SelectQuery<Simplify<{ [TName in keyof T]: T[TName]; }>, {}, MoreThanOneColumnSelected>;
    <T1 extends NmdExpr>(this: void | SelectQuery<{}, {}, NoColumnsSelected>, expr1: T1): SelectQuery<Simplify<NmdExprToRow<T1>>, {}, NamedExpressionNameOf<T1>>;
    <T1_1 extends NmdExpr>(expr1: T1_1): SelectQuery<Simplify<NmdExprToRow<T1_1>>, {}, MoreThanOneColumnSelected>;
    <T1_2 extends NmdExpr, T2 extends NmdExpr>(expr1: T1_2, expr2: T2): SelectQuery<Simplify<NmdExprToRow<T1_2> & NmdExprToRow<T2>>, {}, MoreThanOneColumnSelected>;
    <T1_3 extends NmdExpr, T2_1 extends NmdExpr, T3 extends NmdExpr>(expr1: T1_3, expr2: T2_1, expr3: T3): SelectQuery<Simplify<NmdExprToRow<T1_3> & NmdExprToRow<T2_1> & NmdExprToRow<T3>>, {}, MoreThanOneColumnSelected>;
    <T1_4 extends NmdExpr, T2_2 extends NmdExpr, T3_1 extends NmdExpr, T4 extends NmdExpr>(expr1: T1_4, expr2: T2_2, expr3: T3_1, expr4: T4): SelectQuery<Simplify<NmdExprToRow<T1_4> & NmdExprToRow<T2_2> & NmdExprToRow<T3_1> & NmdExprToRow<T4>>, {}, MoreThanOneColumnSelected>;
    <T1_5 extends NmdExpr, T2_3 extends NmdExpr, T3_2 extends NmdExpr, T4_1 extends NmdExpr, T5 extends NmdExpr>(expr1: T1_5, expr2: T2_3, expr3: T3_2, expr4: T4_1, expr5: T5): SelectQuery<Simplify<NmdExprToRow<T1_5> & NmdExprToRow<T2_3> & NmdExprToRow<T3_2> & NmdExprToRow<T4_1> & NmdExprToRow<T5>>, {}, MoreThanOneColumnSelected>;
    <T1_6 extends NmdExpr, T2_4 extends NmdExpr, T3_3 extends NmdExpr, T4_2 extends NmdExpr, T5_1 extends NmdExpr, T6 extends NmdExpr>(expr1: T1_6, expr2: T2_4, expr3: T3_3, expr4: T4_2, expr5: T5_1, expr6: T6): SelectQuery<Simplify<NmdExprToRow<T1_6> & NmdExprToRow<T2_4> & NmdExprToRow<T3_3> & NmdExprToRow<T4_2> & NmdExprToRow<T5_1> & NmdExprToRow<T6>>, {}, MoreThanOneColumnSelected>;
    <T1_7 extends NmdExpr, T2_5 extends NmdExpr, T3_4 extends NmdExpr, T4_3 extends NmdExpr, T5_2 extends NmdExpr, T6_1 extends NmdExpr, T7 extends NmdExpr>(expr1: T1_7, expr2: T2_5, expr3: T3_4, expr4: T4_3, expr5: T5_2, expr6: T6_1, expr7: T7): SelectQuery<Simplify<NmdExprToRow<T1_7> & NmdExprToRow<T2_5> & NmdExprToRow<T3_4> & NmdExprToRow<T4_3> & NmdExprToRow<T5_2> & NmdExprToRow<T6_1> & NmdExprToRow<T7>>, {}, MoreThanOneColumnSelected>;
    <T1_8 extends NmdExpr, T2_6 extends NmdExpr, T3_5 extends NmdExpr, T4_4 extends NmdExpr, T5_3 extends NmdExpr, T6_2 extends NmdExpr, T7_1 extends NmdExpr, T8 extends NmdExpr>(expr1: T1_8, expr2: T2_6, expr3: T3_5, expr4: T4_4, expr5: T5_3, expr6: T6_2, expr7: T7_1, expr8: T8): SelectQuery<Simplify<NmdExprToRow<T1_8> & NmdExprToRow<T2_6> & NmdExprToRow<T3_5> & NmdExprToRow<T4_4> & NmdExprToRow<T5_3> & NmdExprToRow<T6_2> & NmdExprToRow<T7_1> & NmdExprToRow<T8>>, {}, MoreThanOneColumnSelected>;
    <TColumnName extends never>(this: SelectQuery<{}, {}, NoColumnsSelected>, column1: TColumnName): SelectQuery<Simplify<{ [TName_1 in TColumnName]: {}[TName_1]; }>, {}, TColumnName>;
    <TColumnNames extends never>(...columns: TColumnNames[]): SelectQuery<Simplify<{ [TName_2 in TColumnNames]: {}[TName_2]; }>, {}, MoreThanOneColumnSelected>;
    (this: ErrorMessage<"Expressions must have names. Use expr.as('name').">, ...expr: Expression<any>[]): "Error";
};
