import { FromItem, Row, FromFactor, HardRow } from "../FromFactor";
import { Table } from "../Table";
import { Expression, ExpressionOrInputValue, AllExpression, NamedExpression, NamedExpressionNameOf, MapExpressionOrInputValue } from "../Expressions";
import { Query, SingleColumn, MoreThanOneColumnSelected, NoColumnsSelected } from './Query';
import { NmdExpr, Simplify, NmdExprToRow, ErrorMessage } from "./Common";
import { JoinConditionBuilder } from "./JoinMixin";
import { BooleanType } from "../Types";
export declare function update<T extends Row>(table: FromItem<T> & Table<any, any>): UpdateQuery<T, {}, T, NoColumnsSelected>;
export declare class UpdateQuery<TColumnsToUpdate extends Row, TReturningColumns extends Row, TFromTblCols extends Row, TSingleColumn extends SingleColumn<TReturningColumns>> extends Query<TReturningColumns, TSingleColumn> {
    private readonly _table;
    private _updatedColumns;
    constructor(table: FromItem<TColumnsToUpdate & TFromTblCols> & Table<any, any>);
    getState(): {
        whereCondition: Expression<BooleanType> | undefined;
        updatedColumns: { [TColumnName in keyof TFromTblCols]?: Expression<TFromTblCols[TColumnName]> | undefined; };
        table: FromItem<TColumnsToUpdate> & Table<any, any>;
        from: FromFactor | undefined;
    } & {
        isWithRecursive: boolean;
        withItems: FromItem<any>[];
        selected: (AllExpression<any> | NamedExpression<any, any>)[];
        returningColumns: import("./Query").RowToNmdExprs<TReturningColumns>;
    };
    /**
     * Selects from a table. If previous tables are already specified, they are cross joined.
     * These tables can be used to for condition or value expressions.
     * @param table The table to select from.
     */
    from<TTableColumns extends HardRow>(table: FromItem<TTableColumns>): UpdateQuery<TColumnsToUpdate, TReturningColumns, TTableColumns, TSingleColumn>;
    /**
     * Sets a new value for a column.
     * Use `defaultValue()` to reset a column to its default value.
     *
     * @param column The column to update.
     * @param value The new value for the column.
     */
    set<TColumn extends keyof TColumnsToUpdate>(column: TColumn, value: ExpressionOrInputValue<TColumnsToUpdate[TColumn]>): this;
    /**
     * Sets new values for specified columns.
     * @param obj The columns to update and their new values.
     */
    set(obj: {
        [TColumnName in keyof TColumnsToUpdate]?: ExpressionOrInputValue<TColumnsToUpdate[TColumnName]>;
    }): this;
    /** Selects all columns from the given table. */
    returning<T extends Row>(table: AllExpression<T>): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & {
        [TName in keyof T]: T[TName];
    }>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects a single named expression. */
    returning<T1 extends NmdExpr>(this: UpdateQuery<TColumnsToUpdate, TReturningColumns, TFromTblCols, NoColumnsSelected> | void, expr1: T1): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1>>, TFromTblCols, NamedExpressionNameOf<T1>>;
    /** Selects 1 named expressions. */
    returning<T1 extends NmdExpr>(expr1: T1): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 2 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr>(expr1: T1, expr2: T2): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 3 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 4 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 5 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 6 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 7 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr, T7 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6, expr7: T7): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6> & NmdExprToRow<T7>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects 8 named expressions. */
    returning<T1 extends NmdExpr, T2 extends NmdExpr, T3 extends NmdExpr, T4 extends NmdExpr, T5 extends NmdExpr, T6 extends NmdExpr, T7 extends NmdExpr, T8 extends NmdExpr>(expr1: T1, expr2: T2, expr3: T3, expr4: T4, expr5: T5, expr6: T6, expr7: T7, expr8: T8): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & NmdExprToRow<T1> & NmdExprToRow<T2> & NmdExprToRow<T3> & NmdExprToRow<T4> & NmdExprToRow<T5> & NmdExprToRow<T6> & NmdExprToRow<T7> & NmdExprToRow<T8>>, TFromTblCols, MoreThanOneColumnSelected>;
    /** Selects a single column that is currently in scope. */
    returning<TColumnName extends keyof TFromTblCols>(this: UpdateQuery<TColumnsToUpdate, TReturningColumns, TFromTblCols, NoColumnsSelected>, column1: TColumnName): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & {
        [TName in TColumnName]: TFromTblCols[TName];
    }>, TFromTblCols, TColumnName>;
    /** Selects columns that are currently in scope. */
    returning<TColumnNames extends keyof TFromTblCols>(...columns: TColumnNames[]): UpdateQuery<TColumnsToUpdate, Simplify<TReturningColumns & {
        [TName in TColumnNames]: TFromTblCols[TName];
    }>, TFromTblCols, MoreThanOneColumnSelected>;
    returning(this: ErrorMessage<"Expressions must have names. Use expr.as('name').">, ...expr: Expression<any>[]): "Error";
    protected _whereCondition: Expression<BooleanType> | undefined;
    protected lastFromItem: FromItem<TFromTblCols> | undefined;
    /**
    * Adds where conditions.
    * @param obj The object that defines equals expressions.
    */
    where(obj: Partial<MapExpressionOrInputValue<TFromTblCols>>): this;
    /**
    * Adds where conditions.
    * @param conditions The condition expressions.
    */
    where(...conditions: Expression<BooleanType>[]): this;
    /**
    * Adds negated where conditions.
    */
    whereNot(condition: Expression<BooleanType>, ...conditions: Expression<BooleanType>[]): this;
    protected _from: FromFactor | undefined;
    /**
    * Performs a full join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * for (row r in cur): if (joined has no row that matches r)
    * 	yield row(r, null)
    * for (row j in joined): if (cur has no row that matches j)
    * 	yield row(null, j)
    * ```
    */
    fullJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
    /**
    * Performs a left join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * for (row r in cur): if (joined has no row that matches r)
    * 	yield row(r, null)
    * ```
    */
    leftJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
    /**
    * Performs an inner join on the current query (`cur`) and a specified table (`joined`).
    * These rows are returned:
    * ```
    * for (row r in cur): for (row j in joined that matches r)
    * 	yield row(r, j)
    * ```
    */
    innerJoin<TFromItemColumns extends HardRow>(fromItem: FromItem<TFromItemColumns>): JoinConditionBuilder<TFromItemColumns, this>;
}
