Object.defineProperty(exports, "__esModule", { value: true });
exports.RollbackTransactionStatement = exports.CommitTransactionStatement = exports.StartTransactionStatement = exports.IsolationLevel = void 0;
var IsolationLevel;
(function (IsolationLevel) {
    IsolationLevel[IsolationLevel["Serializable"] = 0] = "Serializable";
    IsolationLevel[IsolationLevel["RepeatableRead"] = 1] = "RepeatableRead";
    IsolationLevel[IsolationLevel["ReadCommitted"] = 2] = "ReadCommitted";
    IsolationLevel[IsolationLevel["ReadUncommitted"] = 3] = "ReadUncommitted";
})(IsolationLevel = exports.IsolationLevel || (exports.IsolationLevel = {}));
function isolationLevelToString(isolationLevel) {
    switch (isolationLevel) {
        case IsolationLevel.Serializable: return "SERIALIZABLE";
        case IsolationLevel.RepeatableRead: return "REPEATABLE READ";
        case IsolationLevel.ReadCommitted: return "READ COMMITTED";
        case IsolationLevel.ReadUncommitted: return "READ UNCOMMITTED";
        default: throw new Error(`Invalid isolation level '${isolationLevel}'.`);
    }
}
class StartTransactionStatement {
    constructor(isolationLevel) {
        this.isolationLevel = isolationLevel;
    }
    toString() {
        const isoLevel = this.isolationLevel ? (" ISOLATION LEVEL " + isolationLevelToString(this.isolationLevel)) : "";
        return `START TRANSACTION${isoLevel}`;
    }
}
exports.StartTransactionStatement = StartTransactionStatement;
class CommitTransactionStatement {
    toString() { return "COMMIT"; }
}
exports.CommitTransactionStatement = CommitTransactionStatement;
class RollbackTransactionStatement {
    toString() { return "ROLLBACK"; }
}
exports.RollbackTransactionStatement = RollbackTransactionStatement;
//# sourceMappingURL=SqlStatement.js.map