import { Row, FromItem, RowToColumns, ColumnsToRow } from "./FromFactor";
import { ExpressionTypeOf } from "./Expressions";
import { AnyType, GetInType } from "./Types";
export interface TableName {
    schema?: string;
    name: string;
}
export declare class Table<TRequiredColumns extends Row, TOptionalColumns extends Row> extends FromItem<TRequiredColumns & TOptionalColumns> {
    readonly $name: TableName;
    $requiredColumns: RowToColumns<TRequiredColumns>;
    $optionalColumns: RowToColumns<TOptionalColumns>;
    constructor($name: TableName, requiredColumns: RowToColumns<TRequiredColumns>, optionalColumns: RowToColumns<TOptionalColumns>);
    toString(): string;
}
export declare type TableRequiredColumns<TTable extends Table<any, any>> = ColumnsToRow<TTable["$requiredColumns"]>;
export declare type TableOptionalColumns<TTable extends Table<any, any>> = ColumnsToRow<TTable["$optionalColumns"]>;
export declare type TableToInRow<TFromItem extends Table<any, any>> = {
    [TName in keyof TableRequiredColumns<TFromItem>]: GetInType<ExpressionTypeOf<TFromItem["$requiredColumns"][TName]>>;
} & {
    [TName in keyof TableOptionalColumns<TFromItem>]?: GetInType<ExpressionTypeOf<TFromItem["$optionalColumns"][TName]>>;
};
export declare type TableCtor<TRequiredColumns extends Row, TOptionalColumns extends Row> = Table<TRequiredColumns, TOptionalColumns> & RowToColumns<TRequiredColumns & TOptionalColumns>;
export interface RowDescription {
    [columnName: string]: AnyType;
}
export declare type RowDescriptionToRow<TColumns extends RowDescription> = TColumns;
/**
 * Declares a table.
 * @param tableName The name of the table.
 * @param requiredColumns Columns that need a value when inserting new rows.
 */
export declare function table<TColumnsWithTypes1 extends RowDescription>(tableName: string | TableName, requiredColumns: TColumnsWithTypes1): TableCtor<RowDescriptionToRow<TColumnsWithTypes1>, {}>;
/**
 * Declares a table.
 * @param tableName The name of the table.
 * @param requiredColumns Columns that need a value when inserting new rows.
 * @param optionalColumns Columns that don't need a value when inserting new rows.
 */
export declare function table<TColumnsWithTypes1 extends RowDescription, TColumnsWithTypes2 extends RowDescription>(tableName: string | TableName, requiredColumns: TColumnsWithTypes1, optionalColumns: TColumnsWithTypes2): TableCtor<RowDescriptionToRow<TColumnsWithTypes1>, RowDescriptionToRow<TColumnsWithTypes2>>;
