Object.defineProperty(exports, "__esModule", { value: true });
exports.table = exports.Table = void 0;
const FromFactor_1 = require("./FromFactor");
const Expressions_1 = require("./Expressions");
const Helpers_1 = require("../Helpers");
class Table extends FromFactor_1.FromItem {
    constructor($name, requiredColumns, optionalColumns) {
        super((0, Helpers_1.combine)(requiredColumns, optionalColumns), false);
        this.$name = $name;
        this.$requiredColumns = requiredColumns;
        this.$optionalColumns = optionalColumns;
    }
    toString() {
        let str = "";
        if (this.$name.schema)
            str = `${this.$name.schema}.`;
        str += this.$name.name;
        return str;
    }
}
exports.Table = Table;
;
function table(tableName, requiredColumns, optionalColumns = {}) {
    const setters = [];
    const toTableColumn = (name, columnType) => {
        if (!columnType)
            throw new Error(`Column '${name}' has no type.`);
        return new Expressions_1.TableColumn(name, columnType, setter => setters.push(setter));
    };
    const reqColumns = (0, Helpers_1.toObject)((0, Helpers_1.objectEntries)(requiredColumns)
        .map(([name, columnType]) => toTableColumn(name, columnType)), i => i.name);
    const optColumns = (0, Helpers_1.toObject)((0, Helpers_1.objectEntries)(optionalColumns)
        .map(([name, columnType]) => toTableColumn(name, columnType)), i => i.name);
    const tblName = (typeof tableName === "string") ? { name: tableName, schema: undefined } : tableName;
    const table = new Table(tblName, reqColumns, optColumns);
    for (const setter of setters)
        setter(table);
    return table;
}
exports.table = table;
//# sourceMappingURL=Table.js.map