import { Row } from "./FromFactor";
export declare type AnyType = Type<any, any, string>;
export declare type MapOutType<TRow extends Row> = {
    [TName in keyof TRow]: GetOutType<TRow[TName]>;
};
export declare type GetInType<TType extends {
    _inType: any;
}> = TType["_inType"];
export declare type GetOutType<TType extends {
    _outType: any;
}> = TType["_outType"];
export declare abstract class Type<TInType, TOutType, TBrand extends string> {
    readonly _brand: TBrand;
    _inType: TInType;
    _outType: TOutType;
    abstract name: string;
    abstract serialize(arg: TInType): string | number | boolean | null;
    abstract deserialize(arg: string | number | boolean): TOutType;
    orNull(): Type<TInType | null, TOutType | null, TBrand | "null">;
}
export declare class NullType<TInType, TOutType, TBrand extends string> extends Type<TInType | null, TOutType | null, TBrand | "null"> {
    private readonly type;
    name: string;
    constructor(type: Type<TInType, TOutType, TBrand>);
    serialize(arg: TInType | null): string | number | boolean | null;
    deserialize(arg: string | number | boolean | null): TOutType | null;
}
export declare class StandaloneNullType extends Type<null, null, "null"> {
    name: string;
    serialize(arg: null): string | number | boolean | null;
    deserialize(arg: string | number | boolean | null): null;
}
export declare const tNull: StandaloneNullType;
export declare class VoidType extends Type<void, void, "void"> {
    name: string;
    serialize(arg: void): string | number | boolean | null;
    deserialize(arg: string | number | boolean): void;
}
export declare const tVoid: VoidType;
export declare type RecordTypeToJson<T extends {
    _brand: string;
    _recordType: {
        [name: string]: any;
    };
    _inType: any;
}> = ({
    record: {
        [TName in keyof T["_recordType"]]: RecordTypeToJson<T["_recordType"][TName]>;
    };
} & {
    [other: string]: GetInType<T>;
})[T["_brand"]];
export declare function tJson<T>(): Json<T>;
export declare class Json<T extends any> extends Type<T, T, "json"> {
    name: string;
    serialize(arg: T): string | number | boolean;
    deserialize(arg: string | number | boolean): T;
}
export declare class Record<T> extends Type<never, string, "record"> {
    name: string;
    _recordType: T;
    serialize(arg: never): string | number | boolean;
    deserialize(arg: string): string;
}
export declare function tRecord<T>(): Record<T>;
export declare class BooleanType extends Type<boolean, boolean, "boolean"> {
    name: string;
    serialize(arg: boolean): string | number | boolean;
    deserialize(arg: string | number | boolean): boolean;
}
export declare const tBoolean: BooleanType;
export declare class TextType extends Type<string, string, "text"> {
    name: string;
    serialize(arg: string): string | number | boolean;
    deserialize(arg: string | number | boolean): string;
}
export declare const tText: TextType;
export declare class StringEnumType<T extends string> extends Type<T, T, "string_enum"> {
    name: string;
    serialize(arg: T): string | number | boolean;
    deserialize(arg: string | number | boolean): T;
}
export declare function tStringEnum<T extends string>(): StringEnumType<T>;
export declare class IntegerEnumType<T extends number> extends Type<T, T, "integer_enum"> {
    name: string;
    serialize(arg: T): string | number | boolean;
    deserialize(arg: string | number | boolean): T;
}
export declare function tIntegerEnum<T extends number>(): IntegerEnumType<T>;
export declare class IntegerType extends Type<number, number, "integer"> {
    name: string;
    serialize(arg: number): string | number | boolean;
    deserialize(arg: string | number | boolean): number;
}
export declare const tInteger: IntegerType;
export declare class BigIntType extends Type<string, string, "bigint"> {
    name: string;
    serialize(arg: string): string | number | boolean;
    deserialize(arg: string | number | boolean): string;
}
export declare const tBigInt: BigIntType;
export declare class NumericType extends Type<number, number, "numeric"> {
    name: string;
    serialize(arg: number): string | number | boolean;
    deserialize(arg: string | number | boolean): number;
}
export declare const tNumeric: NumericType;
export declare class DateType extends Type<Date, Date, "date"> {
    name: string;
    serialize(arg: Date): string | number | boolean;
    deserialize(arg: string | number | boolean): Date;
}
export declare const tDate: DateType;
