Object.defineProperty(exports, "__esModule", { value: true });
exports.tDate = exports.DateType = exports.tNumeric = exports.NumericType = exports.tBigInt = exports.BigIntType = exports.tInteger = exports.IntegerType = exports.tIntegerEnum = exports.IntegerEnumType = exports.tStringEnum = exports.StringEnumType = exports.tText = exports.TextType = exports.tBoolean = exports.BooleanType = exports.tRecord = exports.Record = exports.Json = exports.tJson = exports.tVoid = exports.VoidType = exports.tNull = exports.StandaloneNullType = exports.NullType = exports.Type = void 0;
class Type {
    orNull() {
        return new NullType(this);
    }
}
exports.Type = Type;
class NullType extends Type {
    constructor(type) {
        super();
        this.type = type;
        this.name = "null";
    }
    serialize(arg) {
        if (arg === null)
            return null;
        return this.type.serialize(arg);
    }
    deserialize(arg) {
        if (arg === null)
            return null;
        return this.type.deserialize(arg);
    }
}
exports.NullType = NullType;
class StandaloneNullType extends Type {
    constructor() {
        super(...arguments);
        this.name = "null";
    }
    serialize(arg) {
        if (arg === null)
            return null;
        throw new Error("Can only serialize null.");
    }
    deserialize(arg) {
        if (arg === null)
            return null;
        throw new Error("Can only deserialize null.");
    }
}
exports.StandaloneNullType = StandaloneNullType;
exports.tNull = new StandaloneNullType();
class VoidType extends Type {
    constructor() {
        super(...arguments);
        this.name = "void";
    }
    serialize(arg) { throw new Error("Void type does not support serialization"); }
    deserialize(arg) { throw new Error("Void type does not support deserialization"); }
}
exports.VoidType = VoidType;
exports.tVoid = new VoidType();
//let test: RecordTypeToJson<{ _brand: "record", _recordType: { p2: { _brand: "record", _recordType: { fooa: { _brand: "record", _recordType: { bla: IntegerType } } } } } }> = null!;
//test.p2.fooa.bla;
function tJson() {
    return new Json();
}
exports.tJson = tJson;
class Json extends Type {
    constructor() {
        super(...arguments);
        this.name = "JSON";
    }
    serialize(arg) {
        return JSON.stringify(arg);
    }
    deserialize(arg) {
        if (typeof arg !== "string")
            throw new Error("Arg must be of type string.");
        return JSON.parse(arg);
    }
}
exports.Json = Json;
class Record extends Type {
    constructor() {
        super(...arguments);
        this.name = "RECORD";
    }
    serialize(arg) {
        throw "";
    }
    deserialize(arg) {
        return arg;
    }
}
exports.Record = Record;
function tRecord() {
    return new Record();
}
exports.tRecord = tRecord;
class BooleanType extends Type {
    constructor() {
        super(...arguments);
        this.name = "boolean";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return !!arg;
    }
}
exports.BooleanType = BooleanType;
exports.tBoolean = new BooleanType();
class TextType extends Type {
    constructor() {
        super(...arguments);
        this.name = "text";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return "" + arg;
    }
}
exports.TextType = TextType;
exports.tText = new TextType();
class StringEnumType extends Type {
    constructor() {
        super(...arguments);
        this.name = "text";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return "" + arg;
    }
}
exports.StringEnumType = StringEnumType;
function tStringEnum() {
    return new StringEnumType();
}
exports.tStringEnum = tStringEnum;
class IntegerEnumType extends Type {
    constructor() {
        super(...arguments);
        this.name = "numeric";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return +arg;
    }
}
exports.IntegerEnumType = IntegerEnumType;
function tIntegerEnum() {
    return new IntegerEnumType();
}
exports.tIntegerEnum = tIntegerEnum;
class IntegerType extends Type {
    constructor() {
        super(...arguments);
        this.name = "integer";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return +arg;
    }
}
exports.IntegerType = IntegerType;
exports.tInteger = new IntegerType();
class BigIntType extends Type {
    constructor() {
        super(...arguments);
        this.name = "bigint";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return "" + arg;
    }
}
exports.BigIntType = BigIntType;
exports.tBigInt = new BigIntType();
class NumericType extends Type {
    constructor() {
        super(...arguments);
        this.name = "numeric";
    }
    serialize(arg) {
        return arg;
    }
    deserialize(arg) {
        return +arg;
    }
}
exports.NumericType = NumericType;
exports.tNumeric = new NumericType();
class DateType extends Type {
    constructor() {
        super(...arguments);
        this.name = "date";
    }
    serialize(arg) {
        return arg.toString();
    }
    deserialize(arg) {
        return new Date("" + arg);
    }
}
exports.DateType = DateType;
exports.tDate = new DateType();
//# sourceMappingURL=Types.js.map