Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgreQueryService = void 0;
const Helpers_1 = require("../Helpers");
const PostgreSqlGenerator_1 = require("./PostgreSqlGenerator");
const events_1 = require("hediet-framework/api/events");
const synchronization_1 = require("hediet-framework/api/synchronization");
class PostgreQueryService {
    constructor(pool, sqlGeneratorOptions = {}) {
        this.pool = pool;
        this.onSqlStatementEmitter = new events_1.EventEmitter();
        this.onSqlStatement = this.onSqlStatementEmitter.asEvent();
        this.PostgreExclusiveQueryService = class PostgreExclusiveQueryService {
            constructor(client, queryService) {
                this.client = client;
                this.queryService = queryService;
            }
            async exec(query) {
                return this.queryService.execClient(query, this.client);
            }
        };
        this.sqlGenerator = new PostgreSqlGenerator_1.PostgreSqlGenerator(sqlGeneratorOptions);
    }
    exec(query) {
        return this.execClient(query);
    }
    async execClient(query, client) {
        const { sql: queryText, parameters } = this.sqlGenerator.toSql(query);
        const resultRows = new synchronization_1.Deferred();
        if ((0, Helpers_1.isLoggingEnabled)()) {
            (0, Helpers_1.logFull)("Executing SQL query '%s' with args %o", queryText, parameters);
            (0, Helpers_1.log)("Executing SQL query '%s' with args %o" + (parameters.length > 20 ? "..." : ""), queryText.substr(0, 400) + (queryText.length > 400 ? "..." : ""), parameters.slice(0, 20));
        }
        this.onSqlStatementEmitter.emit(this, { sql: queryText, parameters: parameters, query: query, resultRows: resultRows.value });
        const result = await (client ? client.query(queryText, parameters) : this.pool.query(queryText, parameters));
        let updatedRows = result.rows;
        /*
                if (query instanceof RetrievalQuery) {
                    const cols = query.getState().returningColumns;
                    updatedRows = updatedRows.map(row => {
                        const result = {} as any;
                        for (const prop of row) {
                            const val = row[prop];
                            const type = cols[prop] as AnyType;
                            result[prop] = type.deserialize(val);
                        }
                    });
                }
        */
        resultRows.setValue(updatedRows);
        return updatedRows;
    }
    getExclusiveQueryService(scope) {
        return new Promise((res, rej) => this.pool.connect((err, client, done) => {
            if (err) {
                rej(err);
                return;
            }
            const service = new this.PostgreExclusiveQueryService(client, this);
            scope(service)
                .then(result => {
                done();
                res(result);
            })
                .catch(err => {
                done();
                rej(err);
            });
        }));
    }
}
exports.PostgreQueryService = PostgreQueryService;
//# sourceMappingURL=Postgre.js.map