Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicDispatcher = exports.combine = exports.objectEntries = exports.objectValues = exports.secondWithTypeOfFirst = exports.toObject = exports.warn = exports.log = exports.logFull = exports.isLoggingEnabled = void 0;
const debug = require("debug");
function isLoggingEnabled() {
    return debug.enabled("typed-sql:debug");
}
exports.isLoggingEnabled = isLoggingEnabled;
const sqlDebugLogFull = debug("typed-sql:debug-full");
function logFull(message, ...args) {
    sqlDebugLogFull(message, ...args);
}
exports.logFull = logFull;
const sqlDebugLog = debug("typed-sql:debug");
function log(message, ...args) {
    sqlDebugLog(message, ...args);
}
exports.log = log;
const sqlDebugWarn = debug("typed-sql:warn");
function warn(message, ...args) {
    sqlDebugWarn(message, ...args);
}
exports.warn = warn;
function toObject(item, keySelector, valueSelector) {
    const o = {};
    for (const i of item) {
        o[keySelector(i)] = valueSelector ? valueSelector(i) : i;
    }
    return o;
}
exports.toObject = toObject;
function secondWithTypeOfFirst(t1, t2) {
    return t2;
}
exports.secondWithTypeOfFirst = secondWithTypeOfFirst;
function objectValues(obj) {
    const result = [];
    for (const prop of Object.getOwnPropertyNames(obj)) {
        result.push(obj[prop]);
    }
    return result;
}
exports.objectValues = objectValues;
function objectEntries(obj) {
    const result = [];
    for (const prop of Object.getOwnPropertyNames(obj)) {
        result.push([prop, obj[prop]]);
    }
    return result;
}
exports.objectEntries = objectEntries;
function combine(props, and) {
    const result = {};
    for (const [prop, val] of objectEntries(props)) {
        result[prop] = val;
    }
    for (const [prop, val] of objectEntries(and)) {
        result[prop] = val;
    }
    return result;
}
exports.combine = combine;
class DynamicDispatcher {
    constructor() {
        this.registeredHandlers = new Map();
    }
    register(clazz, handler) {
        this.registeredHandlers.set(clazz, handler);
        return this;
    }
    dispatch(obj, args) {
        let proto = Object.getPrototypeOf(obj);
        while (proto) {
            const handler = this.registeredHandlers.get(proto.constructor);
            if (handler) {
                return handler(obj, args);
            }
            proto = Object.getPrototypeOf(proto);
        }
        throw new Error(`No handler was registered for '${obj}'.`);
    }
}
exports.DynamicDispatcher = DynamicDispatcher;
//# sourceMappingURL=Helpers.js.map