import { UnionQuery } from './AST/Queries/RetrievalQuery';
import { ValuesQuery } from './AST/Queries/ValuesQuery';
import { InsertQuery } from './AST/Queries/InsertQuery';
import { UpdateQuery } from './AST/Queries/UpdateQuery';
import { DeleteQuery } from './AST/Queries/DeleteQuery';
import { SqlStatement } from './AST/SqlStatement';
import { FromFactor, FromItem } from './AST/FromFactor';
import { SelectQuery } from "./AST/Queries/SelectQuery";
import * as Exprs from './AST/Expressions';
import { AnyType } from "./index";
export interface SqlGeneratorOptions {
    shortenColumnNameIfUnambigous?: boolean;
    skipQuotingIfNotRequired?: boolean;
}
export interface ExpressionContext {
    isColumnNameUnambigous(name: string): boolean;
    resolveNamedExpression: boolean;
    context: Context;
}
export interface Context {
    parameters: any[];
}
export declare abstract class SqlGenerator {
    protected readonly options: SqlGeneratorOptions;
    constructor(options?: SqlGeneratorOptions);
    toSql(statement: SqlStatement): {
        sql: string;
        parameters: any[];
    };
    protected transformToSql(statement: SqlStatement, context: Context): string;
    protected createExpressionContext(froms: (FromFactor | undefined)[], context: Context): ExpressionContext;
    protected transformUnionQueyToSql(query: UnionQuery<any, any>, context: Context): string;
    protected transformValuesQueryToSql(query: ValuesQuery<any>, context: Context): string;
    protected transformInsertQueryToSql(query: InsertQuery<any, any, any>, context: Context): string;
    protected transformDeleteQueryToSql(query: DeleteQuery<any, any, any>, context: Context): string;
    protected transformUpdateQueryToSql(query: UpdateQuery<any, any, any, any>, context: Context): string;
    protected transformSelectQueryToSql(query: SelectQuery<any, any, any>, context: Context): string;
    protected toSelectStatementStr(selected: (Exprs.NamedExpression<string, AnyType> | Exprs.AllExpression<object>)[], context: ExpressionContext): string;
    protected quoteSchemaName(name: string): string;
    protected quoteTableName(name: string): string;
    protected quoteColumnName(name: string): string;
    protected abstract quoteSchemaOrTableOrColumnName(name: string): string;
    protected referToFromItem(fromItem: FromItem<any>, includeSchema?: boolean): string;
    private fromFactorDispatcherInitialized;
    private fromFactorDispatcher;
    private fromToSql;
    protected escapeValue(expr: Exprs.ValueExpression<AnyType>, context: ExpressionContext): string;
    private expressionToSql;
    private expressionDispatcherInitialized;
    private expressionDispatcher;
    private expressionToSqlAutoParenthesis;
}
